@@ -1,4 +1,3 @@
-/* $XFree86$ */
 /*
  * Copyright 2003 Alex Deucher.
  *
@@ -212,12 +211,14 @@ RADEONCopyModeNLink(ScrnInfoPtr pScrn, DisplayModePtr dest,
     mode->VSyncEnd += dy;
     mode->VTotal += dy;
 
-     /* Provide a sophisticated fake DotClock in order to trick the vidmode
+     /* Provide a fake VRefresh/DotClock in order to trick the vidmode
       * extension to allow selecting among a number of modes whose merged result
       * looks identical but consists of different modes for CRT1 and CRT2
       */
-    mode->Clock = (((i->Clock >> 3) + i->HTotal) << 16) | ((j->Clock >> 2) + j->HTotal);
-    mode->Clock ^= ((i->VTotal << 19) | (j->VTotal << 3));
+    mode->VRefresh = (float)((i->Clock * 1000.0 / i->HTotal / i->VTotal) * 100 +
+	(j->Clock * 1000.0 / j->HTotal / j->VTotal));
+
+    mode->Clock = (int)(mode->VRefresh * 0.001 * mode->HTotal * mode->VTotal);
 
     if( ((mode->HDisplay * ((pScrn->bitsPerPixel + 7) / 8) * mode->VDisplay) > 
 	(pScrn->videoRam * 1024)) ||
@@ -531,15 +532,27 @@ RADEONGenerateModeList(ScrnInfoPtr pScrn, char* str,
    if(str != NULL) {
       return(RADEONGenerateModeListFromMetaModes(pScrn, str, i, j, srel));
    } else {
-      xf86DrvMsg(pScrn->scrnIndex, X_INFO,
-	"No MetaModes given, linking %s modes by default\n",
-	(srel == radeonClone) ? "largest common" :
-	   (info->NonRect ?
+        if (srel == radeonClone ) {
+           DisplayModePtr p, q, result = NULL;
+
+           xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+                "Clone mode, list all common modes\n");
+           for (p = i; p->next != i; p = p->next)
+                for (q = j; q->next != j; q = q->next)
+                   if ((p->HDisplay == q->HDisplay) &&
+                        (p->VDisplay == q->VDisplay))
+                        result = RADEONCopyModeNLink(pScrn, result, p, q, srel);
+           return result;
+        } else {
+           xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+                "No MetaModes given, linking %s modes by default\n",
+                (info->NonRect ?
 		(((srel == radeonLeftOf) || (srel == radeonRightOf)) ? "widest" :  "tallest")
 		:
 		(((srel == radeonLeftOf) || (srel == radeonRightOf)) ? "widest common" :  "tallest common")) );
-      return(RADEONGenerateModeListFromLargestModes(pScrn, i, j, srel));
-   }
+           return(RADEONGenerateModeListFromLargestModes(pScrn, i, j, srel));
+        }
+    }
 }
 
 void
@@ -2024,14 +2037,7 @@ RADEONSetCursorPositionMerged(ScrnInfoPtr pScrn, int x, int y)
     OUTREG(RADEON_CUR_HORZ_VERT_POSN, (RADEON_CUR_LOCK
 				   | ((xorigin ? 0 : x1) << 16)
 				   | (yorigin ? 0 : y1)));
-#ifdef USE_EXA
-    if (info->useEXA)
-	OUTREG(RADEON_CUR_OFFSET, info->cursor_offset + yorigin * stride);
-#endif /* USE_EXA */
-#ifdef USE_XAA
-    if (!info->useEXA)
-	OUTREG(RADEON_CUR_OFFSET, info->cursor_offset + yorigin * stride);
-#endif /* USE_XAA */
+    OUTREG(RADEON_CUR_OFFSET, info->cursor_offset + yorigin * stride);
 		/* cursor2 */
     OUTREG(RADEON_CUR2_HORZ_VERT_OFF,  (RADEON_CUR2_LOCK
 				    | (xorigin << 16)
@@ -2039,14 +2045,7 @@ RADEONSetCursorPositionMerged(ScrnInfoPtr pScrn, int x, int y)
     OUTREG(RADEON_CUR2_HORZ_VERT_POSN, (RADEON_CUR2_LOCK
 				    | ((xorigin ? 0 : x2) << 16)
 				    | (yorigin ? 0 : y2)));
-#ifdef USE_EXA
-    if (info->useEXA)
-	OUTREG(RADEON_CUR2_OFFSET, info->cursor_offset + yorigin * stride);
-#endif /* USE_EXA */
-#ifdef USE_XAA
-    if (!info->useEXA)
-	OUTREG(RADEON_CUR2_OFFSET, info->cursor_offset + yorigin * stride);
-#endif /* USE_XAA */
+    OUTREG(RADEON_CUR2_OFFSET, info->cursor_offset + yorigin * stride);
 }
 
 /* radeon Xv helpers */
